/*
	File:		Streams.c

	Contains:	Utility routines to stream data into a block of memory

	Written by:	Chris White, Developer Technical Support
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
	
	Change History (most recent first):
	
				  9/28/95	CW		First release

*/



#ifndef __STREAMS__
#define __STREAMS__



#ifndef __TYPES__
	#include <Types.h>
#endif

#ifndef __MEMORY__
	#include <Memory.h>
#endif





typedef struct StreamHeader
{
	Size	blockSize;
	Size	currentSize;
	Size	cursor;
	Size	maxCursor;
	
} tStreamHeader, *tStreamHeaderPtr;



typedef Ptr		tStreamRef;
typedef UInt32	tStreamCursor;



enum
{
	kInvalidStreamRef	= -1,
	kBoundsErr			= -2
};



OSErr NewStream ( tStreamRef* streamRef, Size blockSize );
OSErr DisposeStream ( tStreamRef streamRef );
OSErr SetStreamData ( tStreamRef streamRef, Ptr dataPtr, Size dataSize );
OSErr GetStreamData ( tStreamRef streamRef, Ptr dataPtr, Size dataSize );
OSErr GetStreamDataPtr ( tStreamRef streamRef, Ptr* dataPtr );
Size GetStreamDataSize ( tStreamRef streamRef );
tStreamCursor GetStreamCursor ( tStreamRef streamRef );
OSErr SetStreamCursor ( tStreamRef streamRef, tStreamCursor byteCount );
void ResetStreamCursor ( tStreamRef streamRef );
OSErr CompactStream ( tStreamRef streamRef );









#endif // define __STREAMS__

